const request = require('superagent');
const now = require('nano-time');

/**
 * Store data to influxdb
 * @param {Object} data params
 * @param {string} data.url link to http endpoint
 * @param {string} data.db database name
 * @param {string} data.meas measurement name
 * @param {Object} data.tags tags to add
 * @param {Object} data.values values to send
 * @param {number} data.timestamp time in nanoseconds
 * @param {number} data.retry superagent retries
 * @param {Object|string} data.timeout superagent timeout
 */
const writeToInflux = async data => {
    const retry = data.retry || 3;
    const timeout = data.timeout || {response: 10000, deadline: 20000};
    const time = data.timestamp || now();

    const sendTags = Object.entries(data.tags).map(elem => elem.join('=')).join();
    const sendData = Object.entries(data.values).map(elem => {
        const [, prop] = elem;
        elem[1] = typeof prop === 'string' ? `"${prop}"` : prop;
        return elem.join('=');
    }).join();

    try {
        await request
            .post(`${data.url}/write`)
            .query({db: data.db})
            .send(`${data.meas},${sendTags} ${sendData} ${time}`)
            .retry(retry)
            .timeout(timeout);

    } catch (err) {
        throw new Error([
            'Error sending data to influx:',
            `URL: ${data.url}`,
            `DB: ${data.db}`,
            `Measurement: ${data.meas}`,
            `Tags: ${sendTags}`,
            `Values: ${sendData}`,
            `Timestamp: ${time}`,
            err,
        ].join('\n'));
    }
};

module.exports = writeToInflux;
